package com.muyu.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.base.CategoryBase;
import com.muyu.product.domain.model.CategoryInfoSaveModel;
import com.muyu.product.domain.model.TemplateAttributeGroupModel;
import com.muyu.product.domain.model.TemplateAttributeModel;
import com.muyu.product.domain.req.CategoryInfoEditReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.CategoryParentCommonElementResp;
import com.muyu.product.mapper.CategoryInfoMapper;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * 品类信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class CategoryInfoServiceImpl extends ServiceImpl<CategoryInfoMapper, CategoryInfo> implements CategoryInfoService {


    @Autowired
    private AsCategoryAttributeService asCategoryAttributeService;

    @Autowired
    private AsCategoryAttributeGroupService asCategoryAttributeGroupService;

    @Autowired
    private AsCategoryBrandService asCategoryBrandService;

    @Autowired
    private AsAttributeGroupService asAttributeGroupService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeGroupService attributeGroupService;
    @Autowired
    private CategoryInfoMapper categoryInfoMapper;
    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;
    /**
     * 查询品类信息列表
     *
     * @param categoryInfo 品类信息
     *
     * @return 品类信息
     */
    @Override
    public List<CategoryInfo> list (CategoryInfo categoryInfo) {
        LambdaQueryWrapper<CategoryInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(categoryInfo.getName())) {
            queryWrapper.like(CategoryInfo::getName, categoryInfo.getName());
        }

        if (ObjUtils.notNull(categoryInfo.getImage())) {
            queryWrapper.eq(CategoryInfo::getImage, categoryInfo.getImage());
        }

        if (ObjUtils.notNull(categoryInfo.getParentId())) {
            queryWrapper.eq(CategoryInfo::getParentId, categoryInfo.getParentId());
        }

        if (ObjUtils.notNull(categoryInfo.getStart())) {
            queryWrapper.eq(CategoryInfo::getStart, categoryInfo.getStart());
        }

        if (ObjUtils.notNull(categoryInfo.getIntroduction())) {
            queryWrapper.eq(CategoryInfo::getIntroduction, categoryInfo.getIntroduction());
        }

        return list(queryWrapper);
    }

    /**
     * 保存商品品类
     * @param categoryInfoSaveModel 保存品类模型
     * @return 是否成功
     */
    @Override
    @Transactional
    public boolean save (CategoryInfoSaveModel categoryInfoSaveModel) {
        CategoryInfo categoryInfo = CategoryInfo.saveModelBuild(categoryInfoSaveModel);
        boolean save = this.save(categoryInfo);
        Long categoryInfoId = categoryInfo.getId();
        List<Long> attributeIdList = categoryInfoSaveModel.getAttributeIdList();
        if (attributeIdList != null && !attributeIdList.isEmpty()) {
            asCategoryAttributeService.saveBatch(
                    attributeIdList.stream()
                            .map(attributeId -> AsCategoryAttribute.categoryBuild(categoryInfoId, attributeId))
                            .toList()
            );
        }
        List<Long> attributeGroupIdList = categoryInfoSaveModel.getAttributeGroupIdList();
        if (attributeGroupIdList != null && !attributeGroupIdList.isEmpty()) {
            asCategoryAttributeGroupService.saveBatch(
                    attributeGroupIdList.stream()
                            .map(attributeGroupId -> AsCategoryAttributeGroup.categoryBuild(categoryInfoId, attributeGroupId))
                            .toList()
            );
        }
        List<Long> brandIdList = categoryInfoSaveModel.getBrandIdList();
        if (brandIdList != null && !brandIdList.isEmpty()) {
            asCategoryBrandService.saveBatch(
                    brandIdList.stream()
                            .map(brandId -> AsCategoryBrand.categoryBuild(categoryInfoId, brandId))
                            .toList()
            );
        }
        return save;
    }

    /**
     * 通过品类ID获取父级以上的属性组集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性组集合
     */
    @Override
    public List<AttributeGroup> getAttributeGroup (Long categoryId) {
        List<AttributeGroup> attributeGroupList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttributeGroup::getCategoryId, categoryId);
        List<AsCategoryAttributeGroup> asCategoryAttributeGroupList = asCategoryAttributeGroupService.list(queryWrapper);
        if (asCategoryAttributeGroupList != null && !asCategoryAttributeGroupList.isEmpty()) {
            List<Long> attributeGroupIdList = asCategoryAttributeGroupList.stream()
                    .map(AsCategoryAttributeGroup::getAttributeGroupId)
                    .toList();
            attributeGroupList.addAll(attributeGroupService.listByIds(attributeGroupIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (attributeGroupList.isEmpty()) {
                attributeGroupList.addAll(getAttributeGroup(categoryInfo.getParentId()));
            } else {
                List<AttributeGroup> attributeGroups = getAttributeGroup(categoryInfo.getParentId());
                attributeGroups.forEach(attributeGroup -> {
                    if (!attributeGroupList.contains(attributeGroup)) {
                        attributeGroupList.add(attributeGroup);
                    }
                });
            }
        }
        return attributeGroupList;
    }

    /**
     * 通过品类ID获取父级以上的品牌集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的品牌集合
     */
    @Override
    public List<BrandInfo> getBrand (Long categoryId) {
        List<BrandInfo> brandInfoList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryBrand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryBrand::getCategoryId, categoryId);
        List<AsCategoryBrand> asCategoryBrandList = asCategoryBrandService.list(queryWrapper);
        if (asCategoryBrandList != null && !asCategoryBrandList.isEmpty()) {
            List<Long> brandIdList = asCategoryBrandList.stream()
                    .map(AsCategoryBrand::getBrandId)
                    .toList();
            brandInfoList.addAll(brandInfoService.listByIds(brandIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (brandInfoList.isEmpty()) {
                brandInfoList.addAll(getBrand(categoryInfo.getParentId()));
            } else {
                List<BrandInfo> brandInfos = getBrand(categoryInfo.getParentId());
                brandInfos.forEach(brandInfo -> {
                    if (!brandInfoList.contains(brandInfo)) {
                        brandInfoList.add(brandInfo);
                    }
                });
            }

        }
        return brandInfoList;
    }

    /**
     * 通过品类ID获取父级以上的属性集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性集合
     */
    @Override
    public List<AttributeInfo> getAttribute (Long categoryId) {
        List<AttributeInfo> attributeInfoList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttribute> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttribute::getCategoryId, categoryId);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(queryWrapper);
        if (asCategoryAttributeList != null && !asCategoryAttributeList.isEmpty()) {
            List<Long> attributeIdList = asCategoryAttributeList.stream()
                    .map(AsCategoryAttribute::getAttributeId)
                    .toList();
            attributeInfoList.addAll(attributeInfoService.listByIds(attributeIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (attributeInfoList.isEmpty()) {
                attributeInfoList.addAll(getAttribute(categoryInfo.getParentId()));
            } else {
                List<AttributeInfo> attributeInfos = getAttribute(categoryInfo.getParentId());
                attributeInfos.forEach(attributeInfoQuery -> {
                    if (!attributeInfoList.contains(attributeInfoQuery)) {
                        attributeInfoList.add(attributeInfoQuery);
                    }
                });
            }
        }
        return attributeInfoList;
    }

    @Override
    public <T, AS> List<T> getCommon (Long categoryId, IService<AS> iService, IService<T> bsiService) {
        List<T> list = new ArrayList();
        QueryWrapper<AS> asQueryWrapper = new QueryWrapper<>();
        asQueryWrapper.eq("category_id", categoryId);
        List<AS> asList = iService.list(asQueryWrapper);
        if (asList != null && !asList.isEmpty()) {
            List<Long> baseIdList = asList.stream()
                    .map(as -> {
                        if (as instanceof CategoryBase categoryBase) {
                            return categoryBase.getBaseId();
                        }
                        return null;
                    })
                    .toList();
            list.addAll(bsiService.listByIds(baseIdList));
        }
        CategoryInfo categoryInfo = this.getById(categoryId);
        if (categoryInfo.getParentId() != 0) {
            if (list.isEmpty()) {
                list.addAll(getCommon(categoryInfo.getParentId(), iService, bsiService));
            } else {
                List<T> commonList = getCommon(categoryInfo.getParentId(), iService, bsiService);
                commonList.forEach(common -> {
                    if (!list.contains(common)) {
                        list.add(common);
                    }
                });
            }
        }
        return list;
    }

    /**
     * 回显品类信息和商品属性 ，商品属性组 ，商品品牌
     * @param id
     * @return
     */
    @Override
    public CategoryParentCommonElementResp getCategoryById(Long id) {
        //品类信息
        CategoryInfo info = this.getById(id);
//品类属性中间表
        LambdaQueryWrapper<AsCategoryAttribute> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttribute::getCategoryId,id);
        List<AsCategoryAttribute> attributeList = asCategoryAttributeService.list(queryWrapper);
        List<Long> longs = attributeList.stream().map(AsCategoryAttribute::getAttributeId).toList();
        List<AttributeInfo> attributeInfos = attributeInfoService.listByIds(longs);
//品类属性组中间表
        LambdaQueryWrapper<AsCategoryAttributeGroup> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AsCategoryAttributeGroup::getCategoryId,id);
        List<AsCategoryAttributeGroup> list = asCategoryAttributeGroupService.list(wrapper);
        List<Long> toList = list.stream().map(AsCategoryAttributeGroup::getAttributeGroupId).toList();
        List<AttributeGroup> attributeGroups = attributeGroupService.listByIds(toList);
//品类品牌中间表
        LambdaQueryWrapper<AsCategoryBrand> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AsCategoryBrand::getCategoryId,id);
        List<AsCategoryBrand> asCategoryBrands = asCategoryBrandService.list(lambdaQueryWrapper);
        List<Long> longList = asCategoryBrands.stream().map(AsCategoryBrand::getBrandId).toList();
        List<BrandInfo> brandInfos = brandInfoService.listByIds(longList);
        return CategoryParentCommonElementResp.builder()
                .id(info.getId())
                .name(info.getName())
                .image(info.getImage())
                .start(info.getStart())
                .introduction(info.getIntroduction())
                .attributeInfoList(attributeInfos)
                .attributeGroupList(attributeGroups)
                .brandInfoList(brandInfos).build();
    }

    /**
     * 修改
     * @param id
     * @param categoryInfoEditReq
     */
    @Override
    public void updateCategoryById(Long id, CategoryInfoEditReq categoryInfoEditReq) {
        //修改category_info品类修改
        boolean b = this.updateById(CategoryInfo.editBuild(id, categoryInfoEditReq));
        if (b){
            updateCateInfo(id,categoryInfoEditReq);
            updateCateGroup(id,categoryInfoEditReq);
            updateCateBrand(id,categoryInfoEditReq);
        }
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @Override
    public int removeCateByIds(Long id) {
        CategoryInfo categoryInfo = categoryInfoMapper.selectById(id);
        List<CategoryInfo> categoryInfos=categoryInfoMapper.selByParentId(categoryInfo.getId());
        if (categoryInfos.size()!=0){
            throw new SecurityException("不能删除还有子集");
        }
        List<ProjectInfo> projectInfos=categoryInfoMapper.selectByProjectInfo(categoryInfo.getParentId());
        if (projectInfos.size()!=0){
            throw new SecurityException("不能删除与商品关联");
        }
        int res=categoryInfoMapper.deleteById(id);
        return res;
    }

    private void updateCateBrand(Long id, CategoryInfoEditReq categoryInfoEditReq) {
        //品类品牌中间表as_category_brand
        LambdaQueryWrapper<AsCategoryBrand> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(AsCategoryBrand::getCategoryId,id);
        asCategoryBrandService.remove(lambdaQueryWrapper);
        List<Long> brandIdList = categoryInfoEditReq.getBrandInfoList().stream().map(BrandInfo::getId).toList();
        asCategoryBrandService.saveBatch(
                brandIdList.stream().map(brandId ->AsCategoryBrand.categoryBuild(id,brandId)).toList()
        );
    }

    private void updateCateGroup(Long id, CategoryInfoEditReq categoryInfoEditReq) {
        //删除品类属性组中间表 as_category_attribute_group
        LambdaQueryWrapper<AsCategoryAttributeGroup> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AsCategoryAttributeGroup::getCategoryId,id);
        asCategoryAttributeGroupService.remove(wrapper);
        List<Long> attributeGroupIdList = categoryInfoEditReq.getAttributeGroupList().stream().map(AttributeGroup::getId).toList();
        asCategoryAttributeGroupService.saveBatch(
                attributeGroupIdList.stream().map(aLong -> AsCategoryAttributeGroup.categoryBuild(id,aLong)).toList()
        );
    }

    private void updateCateInfo(Long id, CategoryInfoEditReq categoryInfoEditReq) {
        //删除品类属性中间表
        LambdaQueryWrapper<AsCategoryAttribute> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsCategoryAttribute::getCategoryId,id);
        asCategoryAttributeService.remove(queryWrapper);
        List<Long> longList = categoryInfoEditReq.getAttributeInfoList().stream().map(AttributeInfo::getId).toList();
        asCategoryAttributeService.saveBatch(
                longList.stream().map(aLong -> AsCategoryAttribute.categoryBuild(id,aLong)).toList()
        );
    }

    /**
     * 通过品类ID获取父级以上的属性、属性组、品牌集合
     *
     * @param categoryId 品类ID
     *
     * @return 父级以上的属性、属性组、品牌集合
     */
    @Override
    public CategoryParentCommonElementResp parentCommonElement (Long categoryId) {
        return CategoryParentCommonElementResp.builder()
                .attributeInfoList(getCommon(categoryId, asCategoryAttributeService, attributeInfoService))
                .attributeGroupList(getCommon(categoryId, asCategoryAttributeGroupService, attributeGroupService))
                .brandInfoList(getCommon(categoryId, asCategoryBrandService, brandInfoService))
                .build();
    }

    private void getParentIdListByCateGoryId(List<Long> parentIdList, Long cateGoryId){
        if (cateGoryId.equals(0L)){
            return;
        }
        CategoryInfo categoryInfo = this.getById(cateGoryId); //category_info品类信息表
        if (categoryInfo!=null){
            parentIdList.add(categoryInfo.getId());
            getParentIdListByCateGoryId(parentIdList, categoryInfo.getParentId());
        }
    }

    /**
     * 通过品类ID获取品类共有属性
     *
     * @param cateGoryId 品类ID
     *
     * @return 品类共有属性
     */
    @Override
    public CategoryCommonElementResp getTemplateAttributeByCateGoryId (Long cateGoryId) {
        List<Long> cateGoryIdList = new ArrayList<>();
        getParentIdListByCateGoryId(cateGoryIdList, cateGoryId);   //品类信息id
        // 取出和品类相关联的属性组关系  - 中间表   as_category_attribute_group   and  category_info
        LambdaQueryWrapper<AsCategoryAttributeGroup> asCategoryAttributeGroupLambdaQueryWrapper = new LambdaQueryWrapper<>(){{
            in(AsCategoryAttributeGroup::getCategoryId, cateGoryIdList);
        }};
        // 属性组中间表属性组id
        List<AsCategoryAttributeGroup> categoryAttributeGroupList = asCategoryAttributeGroupService.list(asCategoryAttributeGroupLambdaQueryWrapper);
        //属性组表信息 属性与组中间表
        List<TemplateAttributeGroupModel>  attributeGroupModelList = categoryAttributeGroupList.stream()
                .map(AsCategoryAttributeGroup::getAttributeGroupId)//属性组中间表  取出属性组表去重id-getAttributeGroupId
                .distinct()
                .map(attributeGroupId -> TemplateAttributeGroupModel.attributeGroupBuild(
                        attributeGroupService.getById(attributeGroupId),//用去重后的属性组表id查找  attributeGroup
                        applyAttributeGroupId -> {
                            LambdaQueryWrapper<AsAttributeGroup> asAttributeGroupQueryWrapper = new LambdaQueryWrapper<>() {{
                                eq(AsAttributeGroup::getGroupId, applyAttributeGroupId);
                            }};
                            List<Long> attributeIdList = asAttributeGroupService.list(asAttributeGroupQueryWrapper).stream()//取出属性与组中间表的商品属性id
                                    .map(AsAttributeGroup::getAttributeId)
                                    .toList();
                            if (attributeIdList.isEmpty()) {
                                return new ArrayList<>();
                            }
                            return attributeInfoService.listByIds(attributeIdList).stream()//取出属性与组中间表的商品属性id查找商品属性表
                                    .map(AttributeInfo::buildTemplateModel)
                                    .toList();
                        }
                ))
                .filter(TemplateAttributeGroupModel::isEffective)
                .toList();
        // 查重集合
        Set<Long> attributeIdSet = new HashSet<>(); //  商品属性表id
        // 获取组内所有的属性Id
        if (!attributeGroupModelList.isEmpty()){
            attributeIdSet.addAll(
                    attributeGroupModelList.stream()
                            .flatMap((Function<TemplateAttributeGroupModel, Stream<TemplateAttributeModel>>) templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                            .map(TemplateAttributeModel::getId)
                            .toList()
            );
        }
        //查找商品属性表信息
        List<TemplateAttributeModel> templateAttributeModelList = new ArrayList<>();
        LambdaQueryWrapper<AsCategoryAttribute> categoryAttributeQueryWrapper = new LambdaQueryWrapper<>();
        categoryAttributeQueryWrapper.in(AsCategoryAttribute::getCategoryId, cateGoryIdList);
        List<AsCategoryAttribute> asCategoryAttributeList = asCategoryAttributeService.list(categoryAttributeQueryWrapper);  //品类属性中间表
        if (asCategoryAttributeList != null && !asCategoryAttributeList.isEmpty()){
            List<Long> templateAttributeIdList = asCategoryAttributeList.stream()  //商品属性表id  过滤后的attribute_info品类属性表id
                    .map(AsCategoryAttribute::getAttributeId)
                    .filter(templateAttributeId -> !attributeIdSet.contains(templateAttributeId))
                    .toList();
            templateAttributeModelList = attributeInfoService.listByIds(templateAttributeIdList).stream()// 过滤后的品类属性中间表的商品属性id查找商品属性
                    .map(AttributeInfo::buildTemplateModel)
                    .toList();
            //查询商品属性表
            List<AsProductAttributeInfo> list = asProductAttributeInfoService.listByIds(templateAttributeIdList);
            for (AsProductAttributeInfo asProductAttributeInfo : list) {
                TemplateAttributeModel.builder()
                        .value(asProductAttributeInfo.getValue())
                        .build();
            }
        }
        List<TemplateAttributeModel> attributeModelList = new ArrayList<>();
        if (!templateAttributeModelList.isEmpty()){
            attributeIdSet.addAll(
                    templateAttributeModelList.stream().map(TemplateAttributeModel::getId).toList()//商品属性表id
            );
        }
        LambdaQueryWrapper<AttributeInfo> attributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        attributeInfoLambdaQueryWrapper.notIn(!attributeIdSet.isEmpty(), AttributeInfo::getId, attributeIdSet);
        attributeModelList = attributeInfoService.list(attributeInfoLambdaQueryWrapper).stream().map(AttributeInfo::buildTemplateModel).toList();
        return CategoryCommonElementResp.builder()
                .templateAttributeGroupList(attributeGroupModelList)
                .templateAttributeList(templateAttributeModelList)
                .attributeList(attributeModelList)
                .build();
    }
}
